% Simulation for speckle pattern and its k-space spectrum
% Write by Zhenyu Wan on 2022/04/30
% Modify by Zhenyu Wan on 2022/05/13 for Gaussian distribution with theoretical curves
% Paper Figures 2&3
% Before figures save:
% figure(1) should select 24cm_24cm preset; figure(2) should select 24cm_12cm preset
%% Basic Setting
clear
clf
close all
 
lambda = 1550e-9;        % wavelength (unit: m)
waist0 = 2e-3;           % waist size of Gaussian components (unit: m)
Propagation_z = 0;       % Propagation diatance1 for Gaussian (unit: m)
pixel_size = 2.0e-6;     % pixel spacing (unit: m)
pixel_num = 1000;        % half of screen pixel number of x/y (screen_size = pixel_size *(pixel_num*2+1))
interfering_num = 2000;  % number of superposed Gaussian components
tilt_max = 5*pi/180;     % maximum (or variance) tilt angle for wave front with x/y
zoomin_rate = 1;         % zoom in the calculated k spectrum
k0 = 2*pi/lambda;
%% Construct random function for speckle components
rand_phase = unifrnd(0,2*pi,interfering_num,1);       % random phase of each components
% random distribution: Gaussian distribution in a circular region
rand_Gaussian = abs(randn(interfering_num,1));
rand_tilt_rg = tilt_max*sqrt(rand_Gaussian/max(rand_Gaussian))*max(rand_Gaussian);
rand_tilt_ag = 2*pi*rand(interfering_num,1);
rand_tilt_xg = rand_tilt_rg.*cos(rand_tilt_ag);           % random tilt angle x
rand_tilt_yg = rand_tilt_rg.*sin(rand_tilt_ag);           % random tilt angle y

average_kr2_g_set = sum(((sin(rand_tilt_xg)).^2+(sin(rand_tilt_yg)).^2)/interfering_num);
%% Interfering every tilt Gaussian component
Speckle_E = zeros(pixel_num*2+1,pixel_num*2+1);   % get a speckle pattern matrix for Gaussian
E0 = 1;                                           % the constant term of amplitude of this component
for n = 1:1:interfering_num
    Gaussian_E = Tilt_Gaussian(E0,pixel_num,pixel_size,rand_tilt_xg(n),rand_tilt_yg(n),lambda,waist0,Propagation_z,rand_phase(n));
    Speckle_E = Gaussian_E+Speckle_E;
end

%% Figure 2
figure(1)
set(gcf,'position',[100 100 700 700]);

subplot(2,2,2);             % intensity profile
xy_axis = linspace(-pixel_size*pixel_num,pixel_size*pixel_num,pixel_num*2+1)*1e3;
set(gca,'position', [0.59 0.57 0.4 0.4]);
Speckle_I = (abs(Speckle_E)/max(max(abs(Speckle_E)))).^2;
Speckle_I = imadjust(Speckle_I);   % Enhance image contrast
imagesc(xy_axis,xy_axis,Speckle_I);
set(gca,'FontName','Times New Roman');
colormap(gca, gray); axis square;
axis xy; 
xlabel('$x$ $(mm)$','interpreter','latex','fontsize',16);
ylabel('$y$ $(mm)$','interpreter','latex','fontsize',16);

% Fourier transform of speckle pattern for calculating k spectrum
Speckle_k = fftshift(fft2(Speckle_E));                       % k spectrum (x-y plane)
kx_axis = (-pixel_num:pixel_num)/(2*pixel_size*pixel_num)*2*pi;% x axis of k spectrum
ky_axis = kx_axis;                                             % y axis of k spectrum

% background reduction
Speckle_k = abs(Speckle_k);
K_prob_sort = sort(Speckle_k(:),'descend');
Speckle_k(Speckle_k(:)<K_prob_sort(interfering_num*30)) = Speckle_k(Speckle_k(:)<K_prob_sort(interfering_num*30))/10;

subplot(2,2,4);       % k-spectrum
set(gca,'position', [0.59 0.08 0.4 0.4]);
zoomin = round(pixel_num*(1-zoomin_rate));       % the cut-down shown figure's pixel
Speckle_k_zoomin = abs(Speckle_k(zoomin+1:pixel_num*2+1-zoomin,zoomin+1:pixel_num*2+1-zoomin));  % cut-down k-spectrum
kx_axis_zoomin = kx_axis(zoomin+1:pixel_num*2+1-zoomin);            % cut-down kx axis
ky_axis_zoomin = ky_axis(zoomin+1:pixel_num*2+1-zoomin);            % cut-down ky axis
% imagesc(kx_axis_zoomin/k0,ky_axis_zoomin/k0,abs(Speckle_k_zoomin));

% sampling the figure of speckle_k to be shown more clearly 
sample_rate = 5;
num_zoomin2 = floor((pixel_num*zoomin_rate*2+1)/sample_rate);
Speckle_k_zoomin2 = zeros(num_zoomin2);
for m = 1:1:num_zoomin2
    for n = 1:1:num_zoomin2
        Speckle_k_zoomin2 (m,n) = sum(sum(Speckle_k_zoomin((m-1)*sample_rate+1:1:m*sample_rate,(n-1)*sample_rate+1:1:n*sample_rate)));
    end
end
kx_axis_zoomin2 = kx_axis_zoomin(1:sample_rate:num_zoomin2*sample_rate);
ky_axis_zoomin2 = ky_axis_zoomin(1:sample_rate:num_zoomin2*sample_rate);
imagesc(kx_axis_zoomin2/k0,ky_axis_zoomin2/k0,Speckle_k_zoomin2);
xlabel('$k_x/k_0$','interpreter','latex','fontsize',16);
ylabel('$k_y/k_0$','interpreter','latex','fontsize',16);
set(gca,'FontName','Times New Roman');
set(gca, 'XTick',-0.2:0.2:0.2,'YTick',-0.2:0.2:0.2);
% title('zoom in');
axis xy; colormap(gca, parula); axis square;

subplot(2,2,3);  % random distribution1 - Gaussian
set(gca,'position', [0.08 0.08 0.4 0.4]);
scatter(rand_tilt_xg/pi*180,rand_tilt_yg/pi*180,30,'b','.');
x_axis_zoomin_min = asind(min(kx_axis_zoomin)/k0);
x_axis_zoomin_max = asind(max(kx_axis_zoomin)/k0);
axis([x_axis_zoomin_min,x_axis_zoomin_max,x_axis_zoomin_min,x_axis_zoomin_max]);
xlabel('$\theta_x$ $(^{\circ})$','interpreter','latex','fontsize',16);
ylabel('$\theta_y$ $(^{\circ})$','interpreter','latex','fontsize',16);
set(gca,'FontName','Times New Roman');
% title('(a) Distribution of tilt angles');
grid on; set(gca,'GridLineStyle','--','GridAlpha',0.2);
axis square;

text(-21,75,'\bf(a)','Fontsize',17,'FontName','Times New Roman','color','k');
text(-21,20,'\bf(b)','Fontsize',17,'FontName','Times New Roman','color','k');
text(42,75,'\bf(c)','Fontsize',17,'FontName','Times New Roman','color','w');
text(42,20,'\bf(d)','Fontsize',17,'FontName','Times New Roman','color','w');

subplot(2,2,1);  % Gaussian distribution function
set(gca,'position', [0.08 0.57 0.4 0.4]);
mu_normal=[0 0];
sigma_normal=[(tilt_max/pi*180)^2 0;0 (tilt_max/pi*180)^2];
[x,y] = meshgrid(linspace(x_axis_zoomin_min,x_axis_zoomin_max,80)',linspace(x_axis_zoomin_min,x_axis_zoomin_max,80)');
X = [x(:) y(:)];
z = mvnpdf(X,mu_normal,sigma_normal);
surf(x,y,reshape(z,80,80));
axis([x_axis_zoomin_min,x_axis_zoomin_max,x_axis_zoomin_min,x_axis_zoomin_max,0,0.008]);
xlabel('$\theta_x$ $(^{\circ})$','interpreter','latex','fontsize',16,'rotation',20);
ylabel('$\theta_y$ $(^{\circ})$','interpreter','latex','fontsize',16,'rotation',-20);
zlabel('\rm Probability density','interpreter','latex','fontsize',16);
set(gca,'FontName','Times New Roman');
set(gca,'GridLineStyle','--','GridAlpha',0.2)
shading interp;
axis square;
view(-45,20);

%% k-spectrum data for kr^2 curves
[KX_axis,KY_axis] = meshgrid(kx_axis_zoomin,ky_axis_zoomin); % Cartesian coordinate of k spectrum (zoom in)
KR2_axis = KX_axis.^2+KY_axis.^2;                            % kr^2 of each k spectrum point (zoom in)
K_prob_g = Speckle_k_zoomin/max(max(Speckle_k_zoomin));   % normalized probability of each k spectrum point at z2 (zoom in)
pixel_num_zoomin = round(pixel_num*zoomin_rate);
kr2_axis = linspace(0,max(max(KX_axis))^2,pixel_num_zoomin);   % divide the kr^2 axis evenly
kr2_prob_g = zeros(1,pixel_num_zoomin);                        % preset the array of probability of kr^2

for p = 1:1:pixel_num_zoomin-1           % each ring area along kr & kr^2 axis
    kr2_prob_g(1,p) = sum(sum(K_prob_g(KR2_axis(:)>=kr2_axis(p) & KR2_axis(:)<kr2_axis(p+1))));% sum of k^2 spectrum points within each ring area
end

% filtering the background noise in calculated k-spectrum
sigma = k0*sin(tilt_max);
pixel_num_filter = round(pixel_num_zoomin/max(kr2_axis)*(sigma*5)^2);
kr2_prob_g(pixel_num_filter:pixel_num_zoomin) = kr2_prob_g(pixel_num_filter:pixel_num_zoomin)/100;

figure(2)     % draw kr squared curve of speckle
set(gcf,'position',[300 300 800 400]);
normal_kr2_prob_g = kr2_prob_g/sum(kr2_prob_g(:));  % global normalized probability of kr^2
kr2_axis_k02 = kr2_axis/(k0^2);                     % kr^2/k0^2, axis to be wavelength-independent
% yyaxis left
plot(kr2_axis_k02,normal_kr2_prob_g,'.','MarkerSize',8);
set(gca,'xtick',0:0.02:0.10,'ytick',0:0.002:0.008)
axis([0,max(kr2_axis_k02),0,inf]);
xlabel('$k_r^2/k_0^2$','interpreter','latex','fontsize',18);
ylabel('$\rm Probability$','interpreter','latex','fontsize',18);
% title('Probability distribution of <k_r^2>/k_0^2');
hold on
average_kr2_g = kr2_axis_k02(1:pixel_num_zoomin)*normal_kr2_prob_g(1:pixel_num_zoomin)';
plot([average_kr2_g,average_kr2_g],[0,0.008],'m--','LineWidth',1.5); % draw the average line
text(average_kr2_g,max(normal_kr2_prob_g)*5/6,['$<k_r^2>/k_0^2 =',num2str(average_kr2_g),'$'],'Fontsize',18);
% theoretical kr^2 curve for Gaussian case
sigma = sin(tilt_max);
ideal_kr2_prob_g = sqrt(2*pi*kr2_axis_k02/(sigma^2)).*exp(-kr2_axis_k02/2/(sigma^2));
ideal_kr2_prob_g = ideal_kr2_prob_g/sum(ideal_kr2_prob_g(:));
average_kr2_g_ideal = kr2_axis_k02(1:pixel_num_zoomin)*ideal_kr2_prob_g(1:pixel_num_zoomin)';
% yyaxis right
plot(kr2_axis_k02,ideal_kr2_prob_g,'b','LineWidth',1.5);
set(gca,'xtick',0:0.02:0.10)
axis([0,0.10,0,0.008]);
grid on; set(gca,'GridLineStyle','--','GridAlpha',0.2);
set(gca,'FontName','Times New Roman');
hold off
