% Simulation for k-space spectrum of speckle pattern considering varying tilt
% Write by Zhenyu Wan on 2022/05/14
% Calculate data for Paper Figure 4
%% Basic Setting
clear
clf
close all

calculated_num = 1;      % the calculated number of times (increase 1 for next calculation)

lambda = 1550e-9;        % wavelength (unit: m)
waist0 = 2e-3;           % waist size of Gaussian components (unit: m)
Propagation_z = 0;       % Propagation diatance1 for Gaussian (unit: m)
pixel_size = 2.0e-6;     % pixel spacing (unit: m)
pixel_num = 1000;        % half of screen pixel number of x/y (screen_size = pixel_size *pixel_num)
interfering_num = 1000;  % number of superposed Gaussian components
k0 = 2*pi/lambda;        % total wave vector

tilt_max = 5*pi/180;     % maximum (or variance) tilt angle for wave front with x/y
tilt_num = 10;           % number of calculated different tilt cases
data_address = 'D:\PhD research\Structured Slow Light\1-speckle propagation\numerical calculation\results\Results for paper\matdata\'; % data save address
%% 
tilt_axis = linspace(tilt_max/tilt_num,tilt_max,tilt_num);         % divide the tilt angles
rand_Gaussian = abs(randn(interfering_num,1));
rand_phase = unifrnd(0,2*pi,interfering_num,1);       % random phase of each components
average_kr2_set = zeros(tilt_num,1);
Speckle_k = zeros(pixel_num*2+1,pixel_num*2+1,tilt_num);   % get a speckle pattern matrix
average_kr2 = zeros(tilt_num,1);
average_kr2_ideal = zeros(tilt_num,1);

rand_tilt_rg = tilt_max*sqrt(rand_Gaussian/max(rand_Gaussian))*max(rand_Gaussian);
rand_tilt_ag = 2*pi*rand(interfering_num,1);
rand_tilt_xg = rand_tilt_rg.*cos(rand_tilt_ag);           % random tilt angle x
rand_tilt_yg = rand_tilt_rg.*sin(rand_tilt_ag);           % random tilt angle y
average_kr2_g_set = sum(((k0*sin(rand_tilt_xg)).^2+(k0*sin(rand_tilt_yg)).^2)/interfering_num);

zoomin_rate = ones(tilt_num,1);       % zoom in the calculated k spectrum
% rate = k0*sin(tilt_axis)*5/(pi/pixel_size);  rate(rate>1) =1;
% zoomin_rate = round(rate*100)/100;    % zoom in the calculated k spectrum

normal_kr2_prob = zeros(tilt_num,pixel_num);        % the array of probability of kr^2
ideal_kr2_prob = zeros(tilt_num,pixel_num);         % the array of ideal probability of kr^2
pixel_num_zoomin = round(pixel_num*zoomin_rate);

for m = 1:1:tilt_num
    rand_tilt_r = tilt_axis(m)*sqrt(rand_Gaussian/max(rand_Gaussian))*max(rand_Gaussian);
    rand_tilt_a = 2*pi*rand(interfering_num,1);
    rand_tilt_x = rand_tilt_r.*cos(rand_tilt_a);           % random tilt angle x
    rand_tilt_y = rand_tilt_r.*sin(rand_tilt_a);           % random tilt angle y
    average_kr2_set(m) = sum(((k0*sin(rand_tilt_x)).^2+(k0*sin(rand_tilt_y)).^2)/interfering_num);
    
    [Speckle_k(:,:,m),normal_kr2_prob(m,1:pixel_num_zoomin(m)),average_kr2(m),ideal_kr2_prob(m,1:pixel_num_zoomin(m)),average_kr2_ideal(m)] = Interfering_FFT(rand_tilt_x,rand_tilt_y,zoomin_rate(m),tilt_axis(m),rand_phase);
end

figure(1)     % draw the k(x,y) plane of speckle pattern
kx_axis = (-pixel_num:pixel_num)/(2*pixel_size*pixel_num)*2*pi;  % x axis of k spectrum
ky_axis = kx_axis;                                               % y axis of k spectrum
for m = 1:1:tilt_num    
    Speckle_k(:,:,m) = abs(Speckle_k(:,:,m));    
    subplot(2,ceil(tilt_num/2),m);
    imagesc(kx_axis,ky_axis,Speckle_k(:,:,m));
    title(['@ variance ',num2str(tilt_axis(m)/pi*180),'']);
    axis xy; colormap parula; axis square;
end

figure(2)     % draw kr^2 curve of speckle pattern
zoomin = round(pixel_num*(1-zoomin_rate));                       % the cut-down shown figure's pixel

for m = 1:1:tilt_num
    kx_axis_zoomin = kx_axis(zoomin(m)+1:pixel_num*2+1-zoomin(m));         % cut-down kx axis
    kr2_axis = linspace(0,max(max(kx_axis_zoomin))^2,pixel_num_zoomin(m));    % divide the kr^2 axis evenly
    
    subplot(2,ceil(tilt_num/2),m);
    plot(kr2_axis,normal_kr2_prob(m,1:pixel_num_zoomin(m)),'.');
    axis([0,max(kr2_axis),0,inf]);
    xlabel('kr^2 (rad/m)^2');ylabel('Probability');
    title(['kr^2 probability @ ',num2str(tilt_axis(m)/pi*180),'']);
    hold on
    plot([average_kr2(m),average_kr2(m)],[0,ceil(max(normal_kr2_prob(m,:))*1e2)/1e2],'m--'); % draw the average line 
    text(average_kr2(m),ceil(max(normal_kr2_prob(m,:))*1e2)/1e2*6/7,['kr^2 = ' num2str(average_kr2(m),'%01d')]);
    plot(kr2_axis,ideal_kr2_prob(m,1:pixel_num_zoomin(m)),'b');       % theoretical kr^2 curve
    hold off
end

figure(3)     % draw kr^2 and slowing down via angle variance
subplot(2,1,1);     % kr^2 via angle variance
plot(tilt_axis,average_kr2_ideal,'b',tilt_axis,average_kr2,'r.','markersize',10);
xlabel('Variance of Gaussian random distribution of tilt angles');
ylabel('Expectation values of kr^2');
title('kr^2 via angle variance');
subplot(2,1,2);     % vg/c via angle variance
vg_c = sqrt(1-average_kr2/k0^2)-1;
vg_c_ideal = sqrt(1-average_kr2_ideal/k0^2)-1;
plot(tilt_axis,vg_c_ideal,'b',tilt_axis,vg_c,'r.','markersize',10);
xlabel('Variance of Gaussian random distribution of tilt angles');
ylabel('vg/c-1');
title('vg/c-1 via angle variance');

%% save data
dataname = strcat(data_address,'Gaussian_',num2str(round(lambda*1e9)),' variance angle_',num2str(tilt_max/pi*180),' data',num2str(calculated_num),'.mat');
save(dataname,'tilt_axis','vg_c','vg_c_ideal','average_kr2','average_kr2_ideal','zoomin_rate','normal_kr2_prob','ideal_kr2_prob','rand_Gaussian');
