% Simulation for speckle pattern and its k-space spectrum with adjusted aperture (Beam aperture & NA)
% Write by Zhenyu Wan on 2022/05/19
% Calculate data for Paper Figure 5
%% Basic Setting
clear
clf
close all

calculated_num = 8;         % the calculated number of times (increase 1 for next calculation)
data_address = 'D:\PhD research\Structured Slow Light\1-speckle propagation\numerical calculation\results\Results for paper\matdata\'; % data save address
Aperture_max = 2e-3;        % size of beam aperture (maximum radius)
NA_max = sin(20*pi/180);    % Numerical aperture of the detect system (maximum)
Ap_num = 10;                % number of calculated different tilt cases

lambda = 1550e-9;        % wavelength (unit: m)
waist0 = 2e-3;           % waist of Gaussian components (unit: m)
Propagation_z = 0;       % Propagation diatance1 for Gaussian (unit: m)
pixel_size = 2.0e-6;     % pixel spacing (unit: m)
pixel_num = 1000;        % half of screen pixel number of x/y (screen_size = pixel_size *pixel_num)
interfering_num = 2000;  % number of superposed Gaussian components
tilt_max = 5*pi/180;     % maximum (or variance) tilt angle for wave front with x/y
k0 = 2*pi/lambda;
%% Construct random function for speckle components
rand_phase = unifrnd(0,2*pi,interfering_num,1);       % random phase of each components
% random distribution: Gaussian distribution in a circular region
rand_Gaussian = abs(randn(interfering_num,1));
rand_tilt_rg = tilt_max*sqrt(rand_Gaussian/max(rand_Gaussian))*max(rand_Gaussian);
rand_tilt_ag = 2*pi*rand(interfering_num,1);
rand_tilt_xg = rand_tilt_rg.*cos(rand_tilt_ag);           % random tilt angle x
rand_tilt_yg = rand_tilt_rg.*sin(rand_tilt_ag);           % random tilt angle y

average_kr2_g_set = sum(((sin(rand_tilt_xg)).^2+(sin(rand_tilt_yg)).^2)/interfering_num);
vg_c_set = sqrt(1-average_kr2_g_set)-1;
%% Interfering every tilt Gaussian component (Initial)
Speckle_E = zeros(pixel_num*2+1,pixel_num*2+1);   % get a speckle field matrix
E0 = 1;                                           % the constant term of amplitude of this component
for n = 1:1:interfering_num
    Gaussian_E = Tilt_Gaussian(E0,pixel_num,pixel_size,rand_tilt_xg(n),rand_tilt_yg(n),lambda,waist0,Propagation_z,rand_phase(n));
    Speckle_E = Gaussian_E+Speckle_E;
end
%% Beam aperture
Aperture_size = linspace(Aperture_max/Ap_num,Aperture_max,Ap_num);  % the calculated aperture sizes
kx_axis = (-pixel_num:pixel_num)/(2*pixel_size*pixel_num)*2*pi;     % x axis of k spectrum
ky_axis = kx_axis;                                                  % y axis of k spectrum
[KX_axis,KY_axis] = meshgrid(kx_axis,ky_axis);                      % Cartesian coordinate of k spectrum
KR2_axis = KX_axis.^2+KY_axis.^2;                                   % kr^2 of each k spectrum point
kr2_axis = linspace(0,max(max(KX_axis))^2,pixel_num);               % divide the kr^2 axis evenly
[x,y] = meshgrid(linspace(-pixel_size*pixel_num,pixel_size*pixel_num,pixel_num*2+1)',linspace(-pixel_size*pixel_num,pixel_size*pixel_num,pixel_num*2+1)');
average_kr2_BA = zeros(Ap_num,1);
average_kr2_ideal_BA = zeros(Ap_num,1);
for m = 1:1:Ap_num
    Speckle_E_aperture = Speckle_E;
    Speckle_E_aperture(x(:).^2+y(:).^2 > Aperture_size(m)^2) = 0;  % Low-pass
%     Speckle_E_aperture(x(:).^2+y(:).^2 < Aperture_size(m)^2) = 0;  % High-pass
    Speckle_k_BA = fftshift(fft2(Speckle_E_aperture)); 
    Speckle_k_BA = abs(Speckle_k_BA);
    % background reduction
    K_prob_sort = sort(Speckle_k_BA(:),'descend');
    Speckle_k_BA(Speckle_k_BA(:)<K_prob_sort(interfering_num*600)) = Speckle_k_BA(Speckle_k_BA(:)<K_prob_sort(interfering_num*600))/100;
    K_prob_BA = Speckle_k_BA/max(Speckle_k_BA(:));   % normalized probability of each k spectrum point at z2
    kr2_prob_BA = zeros(1,pixel_num);                        % preset the array of probability of kr^2
    for p = 1:1:pixel_num-1           % each ring area along kr^2 axis
        kr2_prob_BA(1,p) = sum(sum(K_prob_BA(KR2_axis(:)>=kr2_axis(p) & KR2_axis(:)<kr2_axis(p+1))));% sum of k^2 spectrum points within each ring area
    end
    % filtering the background noise in calculated k-spectrum
    sigma = k0*sin(tilt_max);
    pixel_num_filter = round(pixel_num/max(kr2_axis)*(sigma*4)^2);
    kr2_prob_BA(pixel_num_filter:pixel_num) = kr2_prob_BA(pixel_num_filter:pixel_num)/300;
    normal_kr2_prob_BA = kr2_prob_BA/sum(kr2_prob_BA(:));        % global normalized probability of kr^2
    
    kr2_axis_k02 = kr2_axis/(k0^2);                     % kr^2/k0^2, axis to be wavelength-independent
    average_kr2_BA(m) = kr2_axis_k02(1:pixel_num)*normal_kr2_prob_BA(1:pixel_num)';
    
    % theoretical kr^2 curve
    ideal_kr2_prob = sqrt(2*pi*kr2_axis/(sigma^2)).*exp(-kr2_axis/2/(sigma^2));
    ideal_kr2_prob = ideal_kr2_prob/sum(ideal_kr2_prob(:));
    average_kr2_ideal_BA(m) = kr2_axis_k02(1:pixel_num)*ideal_kr2_prob(1:pixel_num)';
end

figure(1)     % draw kr^2 and slowing down via angle variance
subplot(2,1,1);     % kr^2 via Aperture_size
plot(Aperture_size,average_kr2_ideal_BA,'b',Aperture_size,average_kr2_BA,'m.','markersize',10);
axis([0,max(Aperture_size)+min(Aperture_size),min(average_kr2_ideal_BA)*0.5,max(average_kr2_ideal_BA)*2]);
xlabel('Aperture size /m');
ylabel('<k_r^2>/k_0^2');
title('<k_r^2>/k_0^2 via Aperture size');
subplot(2,1,2);     % vg/c via Aperture_size
vg_c_BA = sqrt(1-average_kr2_BA)-1;
vg_c_ideal_BA = sqrt(1-average_kr2_ideal_BA)-1;
plot(Aperture_size,vg_c_ideal_BA,'b',Aperture_size,vg_c_BA,'m.','markersize',10);
axis([0,max(Aperture_size)+min(Aperture_size),min(vg_c_ideal_BA)*2,max(vg_c_ideal_BA)*0.5]);
xlabel('Aperture size /m');
ylabel('vg/c-1');
title('vg/c-1 via Aperture size');

%% Numerical aperture
Speckle_k = fftshift(fft2(Speckle_E));                       % k spectrum (x-y plane) @ Uniform
NA_size = linspace(NA_max/Ap_num,NA_max,Ap_num);             % the calculated NA sizes
[x,y] = meshgrid(kx_axis',ky_axis');
average_kr2_NA = zeros(Ap_num,1);
average_kr2_ideal_NA = zeros(Ap_num,1);
for m = 1:1:Ap_num       % Numerical aperture
    Speckle_k_NA = abs(Speckle_k);
    Speckle_k_NA(x(:).^2+y(:).^2 > (k0*NA_size(m))^2) = 0;  % Low-pass
    % background reduction
    K_prob_sort = sort(Speckle_k_NA(:),'descend');
    Speckle_k_NA(Speckle_k_NA(:)<K_prob_sort(interfering_num*30)) = Speckle_k_NA(Speckle_k_NA(:)<K_prob_sort(interfering_num*30))/6;
    K_prob_NA = Speckle_k_NA/max(Speckle_k_NA(:));   % normalized probability of each k spectrum point at z2
    kr2_prob_NA = zeros(1,pixel_num);                        % preset the array of probability of kr^2
    for p = 1:1:pixel_num-1           % each ring area along kr^2 axis
        kr2_prob_NA(1,p) = sum(sum(K_prob_NA(KR2_axis(:)>=kr2_axis(p) & KR2_axis(:)<kr2_axis(p+1))));% sum of k^2 spectrum points within each ring area
    end
    % filtering the background noise in calculated k-spectrum
    sigma = k0*sin(tilt_max);
    pixel_num_filter = round(pixel_num/max(kr2_axis)*(sigma*5)^2);
    kr2_prob_NA(pixel_num_filter:pixel_num) = kr2_prob_NA(pixel_num_filter:pixel_num)/100;
    normal_kr2_prob_NA = kr2_prob_NA/sum(kr2_prob_NA(:));        % global normalized probability of kr^2
    
    kr2_axis_k02 = kr2_axis/(k0^2);                     % kr^2/k0^2, axis to be wavelength-independent
    average_kr2_NA(m) = kr2_axis_k02(1:pixel_num)*normal_kr2_prob_NA(1:pixel_num)';
    
    % theoretical kr^2 curve
    ideal_kr2_prob_NA = sqrt(2*pi*kr2_axis/(sigma^2)).*exp(-kr2_axis/2/(sigma^2));
    ideal_kr2_prob_NA(kr2_axis(:) > (k0*NA_size(m))^2) = 0;  % Low-pass
    ideal_kr2_prob_NA = ideal_kr2_prob_NA/sum(ideal_kr2_prob_NA(:));
    average_kr2_ideal_NA(m) = kr2_axis_k02(1:pixel_num)*ideal_kr2_prob_NA(1:pixel_num)';
end

figure(2)     % draw kr^2 and slowing down via angle variance
subplot(2,1,1);     % kr^2 via NA_size
plot(NA_size,average_kr2_ideal_NA,'b',NA_size,average_kr2_NA,'m.','markersize',10);
axis([0,max(NA_size)+min(NA_size),0,max(average_kr2_ideal_NA)*1.5]);
xlabel('Numerical Aperture');
ylabel('<k_r^2>/k_0^2');
title('<k_r^2>/k_0^2 via NA');
subplot(2,1,2);     % vg/c via NA_size
vg_c_NA = sqrt(1-average_kr2_NA)-1;
vg_c_ideal_Na = sqrt(1-average_kr2_ideal_NA)-1;
plot(NA_size,vg_c_ideal_Na,'b',NA_size,vg_c_NA,'m.','markersize',10);
axis([0,max(NA_size)+min(NA_size),min(vg_c_ideal_Na)*1.5,0]);
xlabel('Numerical Aperture');
ylabel('vg/c-1');
title('vg/c-1 via NA');
%% save data
dataname = strcat(data_address,'3-AdjAperture @',num2str(round(lambda*1e9)),' Angle_',num2str(tilt_max/pi*180),' w0_',num2str(waist0*1e3),' MaxBA_',num2str(Aperture_max*1e3),' MaxNA_',num2str(NA_max),' data',num2str(calculated_num),'.mat');
save(dataname,'Aperture_size','vg_c_ideal_BA','vg_c_BA','average_kr2_BA','average_kr2_ideal_BA','NA_size','vg_c_ideal_Na','vg_c_NA','average_kr2_NA','average_kr2_ideal_NA','NA_max','Ap_num','rand_Gaussian','interfering_num','lambda','waist0','Aperture_max');
